@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -18,26 +19,27 @@ import (
 	"github.com/containers/storage/pkg/unshare"
 	ispecs "github.com/opencontainers/image-spec/specs-go"
 	rspecs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 )
 
 type globalFlags struct {
-	Debug             bool
-	LogLevel          string
-	Root              string
-	RunRoot           string
-	StorageDriver     string
-	RegistriesConf    string
-	RegistriesConfDir string
-	DefaultMountsFile string
-	StorageOpts       []string
-	UserNSUID         []string
-	UserNSGID         []string
-	CPUProfile        string
-	cpuProfileFile    *os.File
-	MemoryProfile     string
+	Debug                      bool
+	LogLevel                   string
+	Root                       string
+	RunRoot                    string
+	StorageDriver              string
+	RegistriesConf             string
+	RegistriesConfDir          string
+	DefaultMountsFile          string
+	StorageOpts                []string
+	UserNSUID                  []string
+	UserNSGID                  []string
+	CPUProfile                 string
+	cpuProfileFile             *os.File
+	MemoryProfile              string
+	UserShortNameAliasConfPath string
+	CgroupManager              string
 }
 
 var rootCmd = &cobra.Command{
@@ -58,6 +60,7 @@ var rootCmd = &cobra.Command{
 
 var (
 	globalFlagResults globalFlags
+	exitCode          int
 )
 
 func init() {
@@ -76,15 +79,26 @@ func init() {
 		defaultStoreDriverOptions = optionSlice
 	}
 
+	containerConfig, err := config.Default()
+	if err != nil {
+		logrus.Errorf(err.Error())
+		os.Exit(1)
+	}
+	containerConfig.CheckCgroupsAndAdjustConfig()
+
 	cobra.OnInitialize(initConfig)
+	// Disable the implicit `completion` command in cobra.
+	rootCmd.CompletionOptions.DisableDefaultCmd = true
 	//rootCmd.TraverseChildren = true
 	rootCmd.Version = fmt.Sprintf("%s (image-spec %s, runtime-spec %s)", define.Version, ispecs.Version, rspecs.Version)
 	rootCmd.PersistentFlags().BoolVar(&globalFlagResults.Debug, "debug", false, "print debugging information")
 	// TODO Need to allow for environment variable
 	rootCmd.PersistentFlags().StringVar(&globalFlagResults.RegistriesConf, "registries-conf", "", "path to registries.conf file (not usually used)")
 	rootCmd.PersistentFlags().StringVar(&globalFlagResults.RegistriesConfDir, "registries-conf-dir", "", "path to registries.conf.d directory (not usually used)")
+	rootCmd.PersistentFlags().StringVar(&globalFlagResults.UserShortNameAliasConfPath, "short-name-alias-conf", "", "path to short name alias cache file (not usually used)")
 	rootCmd.PersistentFlags().StringVar(&globalFlagResults.Root, "root", storageOptions.GraphRoot, "storage root dir")
 	rootCmd.PersistentFlags().StringVar(&globalFlagResults.RunRoot, "runroot", storageOptions.RunRoot, "storage state dir")
+	rootCmd.PersistentFlags().StringVar(&globalFlagResults.CgroupManager, "cgroup-manager", containerConfig.Engine.CgroupManager, "cgroup manager")
 	rootCmd.PersistentFlags().StringVar(&globalFlagResults.StorageDriver, "storage-driver", storageOptions.GraphDriverName, "storage-driver")
 	rootCmd.PersistentFlags().StringSliceVar(&globalFlagResults.StorageOpts, "storage-opt", defaultStoreDriverOptions, "storage driver option")
 	rootCmd.PersistentFlags().StringSliceVar(&globalFlagResults.UserNSUID, "userns-uid-map", []string{}, "default `ctrID:hostID:length` UID mapping to use")
@@ -122,7 +136,7 @@ func before(cmd *cobra.Command) error {
 	}
 	logrusLvl, err := logrus.ParseLevel(strLvl)
 	if err != nil {
-		return errors.Wrapf(err, "unable to parse log level")
+		return fmt.Errorf("unable to parse log level: %w", err)
 	}
 	logrus.SetLevel(logrusLvl)
 	if globalFlagResults.Debug {
@@ -176,7 +190,7 @@ func shutdownStore(cmd *cobra.Command) error {
 		logrus.Debugf("shutting down the store")
 		needToShutdownStore = false
 		if _, err = store.Shutdown(false); err != nil {
-			if errors.Cause(err) == storage.ErrLayerUsedByContainer {
+			if errors.Is(err, storage.ErrLayerUsedByContainer) {
 				logrus.Infof("failed to shutdown storage: %q", err)
 			} else {
 				logrus.Warnf("failed to shutdown storage: %q", err)
@@ -219,12 +233,13 @@ func main() {
 
 	if err := rootCmd.Execute(); err != nil {
 		if logrus.IsLevelEnabled(logrus.TraceLevel) {
-			fmt.Fprintf(os.Stderr, "%+v\n", err)
+			fmt.Fprintf(os.Stderr, "Error: %+v\n", err)
 		} else {
-			fmt.Fprintf(os.Stderr, "%v\n", err)
+			fmt.Fprintf(os.Stderr, "Error: %v\n", err)
 		}
-		exitCode := cli.ExecErrorCodeGeneric
-		if ee, ok := (errors.Cause(err)).(*exec.ExitError); ok {
+		exitCode = cli.ExecErrorCodeGeneric
+		var ee *exec.ExitError
+		if errors.As(err, &ee) {
 			if w, ok := ee.Sys().(syscall.WaitStatus); ok {
 				exitCode = w.ExitStatus()
 			}
@@ -232,6 +247,6 @@ func main() {
 		if err := shutdownStore(rootCmd); err != nil {
 			logrus.Warnf("failed to shutdown storage: %q", err)
 		}
-		os.Exit(exitCode)
 	}
+	os.Exit(exitCode)
 }
